logutil

package
v1.1.0-beta.0...-3969c85 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 10, 2026 License: Apache-2.0 Imports: 3 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func StatsErrVerboseLogger

func StatsErrVerboseLogger() *zap.Logger

StatsErrVerboseLogger is used to log error messages with verbose details. Do not use it to log the message that is not related to statistics.

func StatsErrVerboseSampleLogger

func StatsErrVerboseSampleLogger() *zap.Logger

StatsErrVerboseSampleLogger with category "stats" is used to log statistics-related messages with verbose error details. It is used to sample the log to avoid too many logs. Do not use it to log the message that is not related to statistics.

func StatsLogger

func StatsLogger() *zap.Logger

StatsLogger with category "stats" is used to log statistic related messages. Do not use it to log the message that is not related to statistics.

func StatsSampleLogger

func StatsSampleLogger() *zap.Logger

StatsSampleLogger with category "stats" is used to log statistic related messages. It is used to sample the log to avoid too many logs. Do not use it to log the message that is not related to statistics.

Types

This section is empty.

Jump to